package com.github.idcarddemo;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.RemoteException;
import android.os.SystemClock;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.huashi.otg.sdk.HSIDCardInfo;
import com.huashi.otg.sdk.HandlerMsg;
import com.huashi.otg.sdk.HsOtgApi;
import com.sunmi.idcardservice.CardCallback;
import com.sunmi.idcardservice.IDCardInfo;
import com.sunmi.idcardservice.IDCardServiceAidl;
import com.sunmi.test.MiFareActivity;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.SoftReference;
import java.nio.ByteBuffer;
import java.text.SimpleDateFormat;
import java.util.Locale;

import butterknife.BindView;
import butterknife.ButterKnife;

public class MainActivity extends AppCompatActivity {


    private static final String mFilePath = Environment.getExternalStorageDirectory().getPath() + "/wltlib";
    private MyHandler mHandler = new MyHandler(this);
    private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
    @BindView(R.id.status)
    TextView mStatus;
    @BindView(R.id.text)
    TextView mText;
    @BindView(R.id.image)
    ImageView mImage;
    private HsOtgApi mApi;
    private boolean isAuto = false;

    public void onMifare(View view) {
        startActivity(new Intent(this, MiFareActivity.class));
    }


    private static class MyHandler extends Handler {

        private SoftReference<MainActivity> mSoftReference;

        MyHandler(MainActivity activity) {
            mSoftReference = new SoftReference<>(activity);
        }

        @Override
        public void handleMessage(Message msg) {
            MainActivity activity = mSoftReference.get();
            if (activity != null) {
                if (msg.what == 99 || msg.what == 100) {
                    activity.mStatus.setText((String) msg.obj);
                } else if (msg.what == HandlerMsg.CONNECT_SUCCESS) {
                    activity.mStatus.setText("连接成功");
                } else if (msg.what == HandlerMsg.CONNECT_ERROR) {
                    activity.mStatus.setText("连接失败");
                } else if (msg.what == HandlerMsg.READ_ERROR) {
                    activity.mStatus.setText("卡认证失败");
                } else if (msg.what == HandlerMsg.READ_SUCCESS) {
                    HSIDCardInfo info = (HSIDCardInfo) msg.obj;
                    activity.parse(info);
                }
            }
        }
    }


    private void parse(HSIDCardInfo info) {
        mStatus.setText("读卡成功");
        byte[] by = info.getFpDate();
        String firstFinger;
        String secondFinger;

        if (by[4] == 0x01) {
            firstFinger = String.format("第一枚指纹,指位:%s,系统质量:%s \n", getFPcode(by[5]), by[6]);
        } else {
            firstFinger = "身份证无指纹\n";
        }

        if (by[512 + 4] == 0x01) {
            secondFinger = String.format("第二枚指纹,指位:%s,系统质量:%s \n", getFPcode(by[512 + 5]), by[512 + 6]);
        } else {
            secondFinger = "身份证无指纹\n";
        }

        mText.setText("姓名: " + info.getPeopleName() + "\n"
                + "性别: " + info.getSex() + "\n"
                + "民族: " + info.getPeople() + "\n"
                + "出生日期: " + sdf.format(info.getBirthDay()) + "\n"
                + "地址: " + info.getAddr() + "\n"
                + "身份证号:" + info.getIDCard() + "\n"
                + "签发机关: " + info.getDepartment() + "\n"
                + "有效期限: " + info.getStrartDate() + "-" + info.getEndDate() + "\n"
                + firstFinger + secondFinger);


        mImage.setImageBitmap(bytes2Bitmap(info.getwltdata()));

        try {
            int net = mApi.Unpack(mFilePath, info.getwltdata());
            if (net != 0) {
                Log.i("bps", "read is error");
                return;
            }
//            FileInputStream fis = new FileInputStream(mFilePath + "/zp.bmp");
//            Bitmap bmp = BitmapFactory.decodeStream(fis);
//            fis.close();
//            mImage.setImageBitmap(bmp);
        } catch (Exception e) {
            Log.i("bps", "got some error ");
            e.printStackTrace();
        }


        Glide.with(this).load(mFilePath + "/zp.bmp").skipMemoryCache(true).diskCacheStrategy(DiskCacheStrategy.NONE).into(mImage);
    }

    private Bitmap bytes2Bitmap(byte[] bytes) {
        Bitmap bitmap = Bitmap.createBitmap(mImage.getWidth(), mImage.getHeight(), Bitmap.Config.ARGB_8888);
        bitmap.copyPixelsFromBuffer(ByteBuffer.wrap(bytes));
        return bitmap;
    }


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        ButterKnife.bind(this);
//        copy("base.dat");
//        copy("license.lic");

        Intent intent = new Intent();
        intent.setPackage("com.sunmi.idcardservice");
        intent.setAction("com.sunmi.idcard");
        bindService(intent, mConnection, Context.BIND_AUTO_CREATE);

    }


    private IDCardServiceAidl mService;
    private ServiceConnection mConnection = new ServiceConnection() {
        @Override
        public void onServiceConnected(ComponentName name, IBinder service) {
            mService = IDCardServiceAidl.Stub.asInterface(service);
        }

        @Override
        public void onServiceDisconnected(ComponentName name) {

        }
    };


    public void connect(View view) {
//        mApi = new HsOtgApi(mHandler, this);
//        int init = mApi.init();
//        if (init == 1) {
//            mStatus.setText("连接成功" + mApi.GetSAMID());
//        } else {
//            mStatus.setText("连接失败");
//        }

        try {
            mService.cancelAutoReading();
        } catch (RemoteException e) {
            e.printStackTrace();
        }

    }

    public void read(View view) {
//        if (mApi.Authenticate(200, 200) != 1) {
//            mStatus.setText("卡认证失败");
//            return;
//        }
//
//        HSIDCardInfo info = new HSIDCardInfo();
//        if (mApi.ReadCard(info, 200, 1300) == 1) {
//            Message message = Message.obtain();
//            message.obj = info;
//            message.what = HandlerMsg.READ_SUCCESS;
//            mHandler.sendMessage(message);
//        }


        try {
            if (mService != null) {
                IDCardInfo cardInfo = mService.readCard();
                if (cardInfo != null) {
                    //显示图片
                    showInfo(cardInfo);
                    Log.i("bps", "read: " + cardInfo.toString());
                } else {
                    Log.i("bps", "card info is null ");
                }
            } else {
                Log.i("bps", "service is null");
            }

        } catch (RemoteException e) {
            Log.i("bps", "got error: " + e.getMessage());
            e.printStackTrace();
        }
    }


    public void auto(View view) {
//        if (isAuto) {
//            mStatus.setText("自动读卡");
//        } else {
//            mStatus.setText("停止读卡");
//            new Thread(new CPUThread()).start();
//        }
//        isAuto = !isAuto;

        try {
            mService.readCardAuto(new CardCallback.Stub() {
                @Override
                public void getCardData(final IDCardInfo info, int code) {
                    if (code == 10) {
                        //读取成功
                        Log.i("bps", "getCardData: " + info.toString());


                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                showInfo(info);
                            }
                        });

                    } else if (code == -10) {
                        //读取失败
                        Log.i("bps", "auto read card did not get data");
                    } else {
                        Log.i("bps", "what is wrong ");
                    }
                }
            });
        } catch (RemoteException e) {
            Log.i("bps", "auto read is error " + e.getMessage());
            e.printStackTrace();
        }
    }

    private void showInfo(IDCardInfo info) {
//        Glide.with(MainActivity.this)
//                .load(info.getImageAddress())
//                .skipMemoryCache(true)
//                .diskCacheStrategy(DiskCacheStrategy.NONE)
//                .into(mImage);
        Glide.with(this).load(mFilePath + "/zp.bmp").skipMemoryCache(true).diskCacheStrategy(DiskCacheStrategy.NONE).into(mImage);
        mText.setText("姓名: " + info.getName() + "\n"
                + "性别: " + info.getGender() + "\n"
                + "民族: " + info.getNation() + "\n"
                + "出生日期: " + info.getBirthDay() + "\n"
                + "地址: " + info.getAddress() + "\n"
                + "身份证号:" + info.getIdCard() + "\n"
                + "签发机关: " + info.getDepartment() + "\n"
                + "有效期限: " + info.getStartDate() + "-" + info.getEndDate() + "\n"
                + "指纹：" + info.getFirstFinger() + "\n" + info.getFirstFinger());
    }


    @Override
    protected void onPause() {
        super.onPause();
        if (mConnection != null) {
            try {
                mService.cancelAutoReading();
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mApi != null) {
            mApi.unInit();
        }
        if (mConnection != null) {
            try {
                mService.cancelAutoReading();
                unbindService(mConnection);
            } catch (RemoteException e) {
                e.printStackTrace();
            }
            mService = null;
        }
    }

    public class CPUThread extends Thread {
        @Override
        public void run() {

            Message msg;
            while (isAuto) {
                if (mApi.Authenticate(200, 200) != 1) {
                    msg = Message.obtain();
                    msg.what = HandlerMsg.READ_ERROR;
                    mHandler.sendMessage(msg);
                } else {
                    HSIDCardInfo info = new HSIDCardInfo();
                    if (mApi.ReadCard(info, 200, 1300) == 1) {
                        msg = Message.obtain();
                        msg.obj = info;
                        msg.what = HandlerMsg.READ_SUCCESS;
                        mHandler.sendMessage(msg);
                    }
                }

                SystemClock.sleep(300);
            }

        }
    }

    /**
     * 指纹 指位代码
     *
     * @param code
     * @return
     */
    String getFPcode(int code) {
        switch (code) {
            case 11:
                return "右手拇指";
            case 12:
                return "右手食指";
            case 13:
                return "右手中指";
            case 14:
                return "右手环指";
            case 15:
                return "右手小指";
            case 16:
                return "左手拇指";
            case 17:
                return "左手食指";
            case 18:
                return "左手中指";
            case 19:
                return "左手环指";
            case 20:
                return "左手小指";
            case 97:
                return "右手不确定指位";
            case 98:
                return "左手不确定指位";
            case 99:
                return "其他不确定指位";
            default:
                return "未知";
        }
    }


    private void copy(String fileName) {
        FileOutputStream out = null;
        InputStream in = null;
        File file = new File(mFilePath);
        if (!file.exists()) {
            file.mkdir();
        }

        File f = new File(mFilePath + "/" + fileName);

        if (f.exists() && f.length() > 0) {
            return;
        }

        try {
            byte[] buffer = new byte[1024];
            out = new FileOutputStream(f);
            in = getAssets().open(fileName);
            while (in.read(buffer) != -1) {
                out.write(buffer);
            }

            out.flush();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (out != null) {
                    out.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }

            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

        }

    }
}
